'#Include Once "xfont.bi"

#define GAMMA 1.3
#include Once "windows.bi"
#include Once "fbgfx.bi"
#Include Once "crt/string.bi"
#include Once "string.bi"
#include Once "crt.bi"

Type LayoutProperties

    LayoutName as String
    LayoutSizeX as Integer
    LayoutSizeY as Integer
    LayoutColor as UInteger
    NumberItems as Integer

End Type


Type ItemProperties

    ItemName as String
    ItemType as Integer
    ItemPosX as Integer
    ItemPosY as Integer
    ItemSize as Integer
    ItemColor as UInteger
    ItemColor1 as UInteger
    ItemColor2 as UInteger
    ItemColor3 as UInteger
    ItemColor4 as UInteger
    ItemFName as String
    ItemMinValue as Single
    ItemMaxValue as Single
    ItemText as String
    ItemChar as String
    ItemMinDeg as Single
    ItemMaxDeg as Single
    ItemTipWidth as Integer
    ItemBackWidth as Integer
    ItemImagePtr As Any Ptr
    
End Type

#define MAXLAYOUTS 20
#define MAXITEMS 100

#define TYPE_STATIC_TEXT 10
#define TYPE_NEEDLE 11
#define TYPE_BMP_IMAGE 12
#define TYPE_LEVEL_METER 13
#define TYPE_BAR_METER 14
#define TYPE_FORMATTED_TEXT 15

Dim Shared Layout(MAXLAYOUTS) as LayoutProperties
Dim Shared Item(MAXLAYOUTS,MAXITEMS) as ItemProperties
Dim Shared NumLayouts as Integer

Dim Shared Speed as integer
Dim Shared RPM as Integer
Dim Shared Value_Gear as Integer

enum TextStyle
  FS_NONE = 0
  FS_BOLD = 2
  FS_ITALIC = 4  
  FS_ANTIALIAS = 8
  FS_BLUR = 16
end enum



declare sub DrawFont(   BUFFER as any ptr=0, _
                        POSX as integer, _ 
                        POSY as integer, _
                        FTEXT as string, _ 
                        FNAME as string, _
                        FSIZE as integer, _
                        FCOLOR as uinteger=rgb(255,255,255), _
                        FSTYLE as integer=0, _
                        CHARSET as integer=DEFAULT_CHARSET)
 



Dim i as integer
Dim Shared EndPosX as integer

'' A function that creates an image buffer with the same 
'' dimensions as a BMP image, and loads a file into it.

'Const NULL As Any Ptr = 0

Function LoadBMP( ByRef filename As Const String ) As Any Ptr

    Dim As Integer filenum, bmpwidth, bmpheight
    Dim As Any Ptr img

    '' open BMP file
    filenum = FreeFile()
    If Open( filename For Binary Access Read As #filenum ) <> 0 Then Return NULL

        '' retrieve BMP dimensions
        Get #filenum, 19, bmpwidth
        Get #filenum, 23, bmpheight

    Close #filenum

    '' create image with BMP dimensions
    img = ImageCreate( bmpwidth, Abs(bmpheight) )

    If img = NULL Then Return NULL

    '' load BMP file into image buffer
    If BLoad( filename, img ) <> 0 Then ImageDestroy( img ): Return NULL

    Return img

End Function




' ********************************************************************
' * Read the custom screen layouts file and report any errors found.
' *
' ********************************************************************

sub ReadLayouts

    Dim As Integer FileID
    Dim As String NextLine
    Dim As Integer NumItems
    Dim R as Integer
    Dim G as Integer
    Dim B as Integer
    Dim TempString as String
    Dim TempString2 as String

    FileID = FreeFile ()
    Open "Layouts.ini" For Input As #FileID

    NumLayouts=0
    NumItems=0

    While Not(Eof(FileID))

        Line Input #FileID, NextLine

        TempString=NextLine    ' In case we need the original string
        NextLine=UCase(Trim(NextLine))
        if left(NextLine,1) = "#" then
            ' Comment Line - Ignore

        elseif left(NextLine,7) = "LAYOUT=" then
            NumLayouts = NumLayouts+1
            if NumLayouts>MAXLAYOUTS then
                'Message the user
                'StopAndExit
            else
                Layout(NumLayouts).LayoutName=right(NextLine,len(NextLine)-7)
                Layout(NumLayouts).LayoutSizeX=1024
                Layout(NumLayouts).LayoutSizeY=576
                Layout(NumLayouts).LayoutColor=RGB(0,0,0)
                Layout(NumLayouts).NumberItems=0
                NumItems=0
               
            end if                                    

        elseif left(NextLine,6) = "SIZEX=" then
            Layout(NumLayouts).LayoutSizeX=val(right(NextLine,Len(NextLine)-6))

        elseif left(NextLine,6) = "SIZEY=" then
            Layout(NumLayouts).LayoutSizeY=val(right(NextLine,Len(NextLine)-6))

        elseif left(NextLine,5) = "ITEM=" then
            NumItems=NumItems+1
            if NumItems>MAXITEMS then
                'Message the user
                'StopAndExit
            else
                Layout(NumLayouts).NumberItems=NumItems
                Item(NumLayouts,NumItems).ItemName=right(NextLine,len(NextLine)-5)
                Item(NumLayouts,NumItems).ItemType=0
                Item(NumLayouts,NumItems).ItemPosX=0
                Item(NumLayouts,NumItems).ItemPosY=0
                Item(NumLayouts,NumItems).ItemSize=0
                Item(NumLayouts,NumItems).ItemColor=0
                Item(NumLayouts,NumItems).ItemColor1=0
                Item(NumLayouts,NumItems).ItemColor2=0
                Item(NumLayouts,NumItems).ItemColor3=0
                Item(NumLayouts,NumItems).ItemColor4=0
                Item(NumLayouts,NumItems).ItemFName=""

            end if

        elseif left(NextLine,9) = "TYPE=TEXT" then
            Item(NumLayouts,NumItems).ItemType=TYPE_STATIC_TEXT
        elseif left(NextLine,11) = "TYPE=NEEDLE" then
            Item(NumLayouts,NumItems).ItemType=TYPE_NEEDLE
        elseif left(NextLine,10) = "TYPE=IMAGE" then
            Item(NumLayouts,NumItems).ItemType=TYPE_BMP_IMAGE
        elseif left(NextLine,8) = "TYPE=LED" then
            Item(NumLayouts,NumItems).ItemType=TYPE_LEVEL_METER
        elseif left(NextLine,10) = "TYPE=VALUE" then
            Item(NumLayouts,NumItems).ItemType=TYPE_FORMATTED_TEXT
            
        elseif left(NextLine,5) = "POSX=" then
            Item(NumLayouts,NumItems).ItemPosX=val(right(NextLine,Len(NextLine)-5))

        elseif left(NextLine,5) = "POSY=" then
            Item(NumLayouts,NumItems).ItemPosY=val(right(NextLine,Len(NextLine)-5))

        elseif left(NextLine,5) = "SIZE=" then
            Item(NumLayouts,NumItems).ItemSize=val(right(NextLine,Len(NextLine)-5))

        elseif left(NextLine,7) = "LENGTH=" then
            Item(NumLayouts,NumItems).ItemSize=val(right(NextLine,Len(NextLine)-7))

        elseif left(NextLine,6) = "COLOR=" then
            TempString=right(NextLine,Len(NextLine)-6)
            TempString2=""
            R=0
            G=0
            B=0
            while(left(TempString,1)<>",") and (TempString<>"")
               TempString2=TempString2 + left(TempString,1)
               TempString=right(TempString,len(TempString)-1)
            wend
            R=val(trim(TempString2))
            TempString=right(TempString,len(TempString)-1)
            TempString2=""
            while(left(TempString,1)<>",") and (TempString<>"")
               TempString2=TempString2 + left(TempString,1)
               TempString=right(TempString,len(TempString)-1)
            wend
            G=val(trim(TempString2))
            TempString=right(TempString,len(TempString)-1)
            TempString2=""
            while(left(TempString,1)<>",") and (TempString<>"")
               TempString2=TempString2 + left(TempString,1)
               TempString=right(TempString,len(TempString)-1)
            wend
            B=val(trim(TempString2))
            Item(NumLayouts,NumItems).ItemColor=RGB(R,G,B)
        elseif left(NextLine,7) = "COLOR1=" then
            TempString=right(NextLine,Len(NextLine)-7)
            TempString2=""
            R=0
            G=0
            B=0
            while(left(TempString,1)<>",") and (TempString<>"")
               TempString2=TempString2 + left(TempString,1)
               TempString=right(TempString,len(TempString)-1)
            wend
            R=val(trim(TempString2))
            TempString=right(TempString,len(TempString)-1)
            TempString2=""
            while(left(TempString,1)<>",") and (TempString<>"")
               TempString2=TempString2 + left(TempString,1)
               TempString=right(TempString,len(TempString)-1)
            wend
            G=val(trim(TempString2))
            TempString=right(TempString,len(TempString)-1)
            TempString2=""
            while(left(TempString,1)<>",") and (TempString<>"")
               TempString2=TempString2 + left(TempString,1)
               TempString=right(TempString,len(TempString)-1)
            wend
            B=val(trim(TempString2))
            Item(NumLayouts,NumItems).ItemColor1=RGB(R,G,B)
        elseif left(NextLine,7) = "COLOR2=" then
            TempString=right(NextLine,Len(NextLine)-7)
            TempString2=""
            R=0
            G=0
            B=0
            while(left(TempString,1)<>",") and (TempString<>"")
               TempString2=TempString2 + left(TempString,1)
               TempString=right(TempString,len(TempString)-1)
            wend
            R=val(trim(TempString2))
            TempString=right(TempString,len(TempString)-1)
            TempString2=""
            while(left(TempString,1)<>",") and (TempString<>"")
               TempString2=TempString2 + left(TempString,1)
               TempString=right(TempString,len(TempString)-1)
            wend
            G=val(trim(TempString2))
            TempString=right(TempString,len(TempString)-1)
            TempString2=""
            while(left(TempString,1)<>",") and (TempString<>"")
               TempString2=TempString2 + left(TempString,1)
               TempString=right(TempString,len(TempString)-1)
            wend
            B=val(trim(TempString2))
            Item(NumLayouts,NumItems).ItemColor2=RGB(R,G,B)
        elseif left(NextLine,7) = "COLOR3=" then
            TempString=right(NextLine,Len(NextLine)-7)
            TempString2=""
            R=0
            G=0
            B=0
            while(left(TempString,1)<>",") and (TempString<>"")
               TempString2=TempString2 + left(TempString,1)
               TempString=right(TempString,len(TempString)-1)
            wend
            R=val(trim(TempString2))
            TempString=right(TempString,len(TempString)-1)
            TempString2=""
            while(left(TempString,1)<>",") and (TempString<>"")
               TempString2=TempString2 + left(TempString,1)
               TempString=right(TempString,len(TempString)-1)
            wend
            G=val(trim(TempString2))
            TempString=right(TempString,len(TempString)-1)
            TempString2=""
            while(left(TempString,1)<>",") and (TempString<>"")
               TempString2=TempString2 + left(TempString,1)
               TempString=right(TempString,len(TempString)-1)
            wend
            B=val(trim(TempString2))
            Item(NumLayouts,NumItems).ItemColor3=RGB(R,G,B)
        elseif left(NextLine,7) = "COLOR4=" then
            TempString=right(NextLine,Len(NextLine)-7)
            TempString2=""
            R=0
            G=0
            B=0
            while(left(TempString,1)<>",") and (TempString<>"")
               TempString2=TempString2 + left(TempString,1)
               TempString=right(TempString,len(TempString)-1)
            wend
            R=val(trim(TempString2))
            TempString=right(TempString,len(TempString)-1)
            TempString2=""
            while(left(TempString,1)<>",") and (TempString<>"")
               TempString2=TempString2 + left(TempString,1)
               TempString=right(TempString,len(TempString)-1)
            wend
            G=val(trim(TempString2))
            TempString=right(TempString,len(TempString)-1)
            TempString2=""
            while(left(TempString,1)<>",") and (TempString<>"")
               TempString2=TempString2 + left(TempString,1)
               TempString=right(TempString,len(TempString)-1)
            wend
            B=val(trim(TempString2))
            Item(NumLayouts,NumItems).ItemColor4=RGB(R,G,B)

        elseif (left(NextLine,9) = "FONTNAME=") then
            Item(NumLayouts,NumItems).ItemFName=right(trim(TempString),Len(NextLine)-9)

        elseif (left(NextLine,9) = "FILENAME=") then
            Item(NumLayouts,NumItems).ItemFName=right(trim(TempString),Len(NextLine)-9)
            Item(NumLayouts,NumItems).ItemImagePtr=LoadBMP( Item(NumLayouts,NumItems).ItemFName )
            If Item(NumLayouts,NumItems).ItemImagePtr = NULL Then
                Print "bmp_load failed"
            end if

        elseif left(NextLine,9) = "MINVALUE=" then
            Item(NumLayouts,NumItems).ItemMinValue=val(right(NextLine,Len(NextLine)-9))

        elseif left(NextLine,9) = "MAXVALUE=" then
            Item(NumLayouts,NumItems).ItemMaxValue=val(right(NextLine,Len(NextLine)-9))

        elseif left(NextLine,5) = "TEXT=" then
            Item(NumLayouts,NumItems).ItemText=right(trim(TempString),Len(NextLine)-5)

        elseif left(NextLine,7) = "FORMAT=" then
            Item(NumLayouts,NumItems).ItemText=right(trim(TempString),Len(NextLine)-7)

        elseif left(NextLine,9) = "COLORMAP=" then
            Item(NumLayouts,NumItems).ItemText=trim(right(NextLine,Len(NextLine)-9))

        elseif left(NextLine,5) = "CHAR=" then
            Item(NumLayouts,NumItems).ItemChar=mid(trim(TempString),6,1)

        elseif left(NextLine,7) = "MINDEG=" then
            Item(NumLayouts,NumItems).ItemMinDeg=val(right(NextLine,Len(NextLine)-7))

        elseif left(NextLine,7) = "MAXDEG=" then
            Item(NumLayouts,NumItems).ItemMaxDeg=val(right(NextLine,Len(NextLine)-7))

        elseif left(NextLine,9) = "TIPWIDTH=" then
            Item(NumLayouts,NumItems).ItemTipWidth=val(right(NextLine,Len(NextLine)-9))

        elseif left(NextLine,10) = "BACKWIDTH=" then
            Item(NumLayouts,NumItems).ItemBackWidth=val(right(NextLine,Len(NextLine)-10))

        else

                         '           Message the User that the line x was not recognised....

        end if

    Wend


end sub




'sub LoadImage(ByRef FileName as String, ImageNum as integer, SizeX as integer, SizeY as integer) 
'    Dim myImage(100) As Any Ptr = ImageCreate( 48, 48 )
'    BLoad "picture.bmp", myImage
'end sub
'Put (10,10), myImage


' ********************************************************************
' * Display some static text on the display.
' *
' ********************************************************************

sub StaticText(PosX As Integer, PosY As Integer, Size As Integer, ThisFont as String, ByRef FontColor as UInteger, ByRef ThisText As String)
    
    DrawFont(,PosX,PosY,ThisText,ThisFont,Size,FontColor,FS_NONE)

' CHECK OUT
' http://ext.freebasic.net/dev-docs/

end sub



' ********************************************************************
' * Display a Font Bar on the display.
' *
' ********************************************************************

sub FontBar(PosX as Integer, PosY as Integer, Size as Integer, ThisFont as String, ByRef Color1 as UInteger, ByRef Color2 as UInteger, ByRef Color3 as UInteger, ByRef Color4 as UInteger, Char as integer, ColorMap as String, MinValue as Single, MaxValue as Single, CurrentValue as Single)

'?? colors ??
'?? flash if greater than MaxValue ??


    Dim OnThreshold as Single
    Dim NumLEDs as integer
    Dim i as integer
'    Dim CharWidth as integer
    Dim LEDColor as integer
    Dim OnColor as UInteger
    Dim OffColor as UInteger

    NumLEDs = len(ColorMap)

'    CharWidth=20
    EndPosX=PosX

    for i=0 to NumLEDs-1
        OnThreshold = MinValue + (i * (MaxValue-MinValue) / NumLEDs)
        LEDColor = valint(mid(ColorMap,i+1,1))
        if LEDColor = 1 then
            OnColor = Color1
            OffColor = Color1/5
        elseif LEDColor = 2 then
            OnColor = Color2
            OffColor = Color2/5
        elseif LEDColor = 3 then
            OnColor = Color3
            OffColor = Color3/5
        else 
            OnColor = Color4
            OffColor = Color4/5
        end if

        if CurrentValue > OnThreshold Then

            DrawFont(,EndPosX,PosY,chr(Char),ThisFont,Size,OnColor,FS_NONE)  ' ON COLOR - C1,C2,C3, or C4
 
        else

            DrawFont(,EndPosX,PosY,chr(Char),ThisFont,Size,OffColor,FS_NONE)  ' OFF COLOR - C1,C2,C3, or C4

        end if

     next i

end sub

 
sub FormattedValue(PosX as Integer, PosY as Integer, Size as Integer, ThisFont as String, ByRef FontColor as UInteger, ByRef FormatString as String, CurrentValue as Single)

    Dim FormattedOutput as String
    
    FormattedOutput = format(CurrentValue,FormatString)

    DrawFont(,PosX,PosY,FormattedOutput,ThisFont,Size,FontColor,FS_NONE)

end sub


sub Needle(CenterX as Integer,CenterY as Integer, Length as Integer, TipWidth as Integer, BackWidth as Integer, ByRef NeedleColor as UInteger, MinDeg as Single, MaxDeg as Single, MinValue as Single, MaxValue as Single, CurrentValue as Single)
    Const ToRadians As Double = 3.1415926535897932 / 180.0

    Dim NormalisedValue as Single
    Dim GaugeDegrees as Single
    Dim CosValue as Single
    Dim SinValue as Single
    Dim TipX1 as Integer
    Dim TipY1 as Integer
    Dim TipX2 as Integer
    Dim TipY2 as Integer
    Dim BackX1 as Integer
    Dim BackY1 as Integer
    Dim BackX2 as Integer
    Dim BackY2 as Integer
'    Dim TipWidth as Integer
'    Dim BackWidth as Integer
    
    NormalisedValue = (CurrentValue - MinValue) / (MaxValue - MinValue)
    if NormalisedValue < 0.0 then NormalisedValue = 0.0
    if NormalisedValue > 1.0 then NormalisedValue = 1.0
    if MinDeg > MaxDeg then 
        GaugeDegrees = ((360.0 - MinDeg + MaxDeg) * NormalisedValue) + MinDeg
    else
        GaugeDegrees = ((MaxDeg - MinDeg) * NormalisedValue) + MinDeg
    end if
    if GaugeDegrees > 360.0 then 
        GaugeDegrees = GaugeDegrees - 360.0
'    else
'        GaugeDegrees = ((MaxDeg - MinDeg) * NormalisedValue) + MinDeg
    end if

    CosValue = cos(GaugeDegrees * ToRadians)
    SinValue = sin(GaugeDegrees * ToRadians)
    TipX1 = CenterX - (TipWidth * CosValue - Length * SinValue) 
    TipY1 = CenterY - (TipWidth * SinValue + Length * CosValue )
    TipX2 = CenterX - (-TipWidth * CosValue - Length * SinValue )
    TipY2 = CenterY - (-TipWidth * SinValue + Length * CosValue)
    BackX1 = CenterX - (BackWidth * CosValue + Length * SinValue * 0.1)
    BackY1 = CenterY - (BackWidth * SinValue - Length * CosValue * 0.1)
    BackX2 = CenterX - (-BackWidth * CosValue + Length * SinValue * 0.1)
    BackY2 = CenterY - (-BackWidth * SinValue - Length * CosValue * 0.1)

    Line (TipX1,TipY1) - (TipX2,TipY2), NeedleColor
    Line - (BackX2,BackY2), NeedleColor 
    Line - (BackX1,BackY1), NeedleColor
    Line - (TipX1,TipY1), NeedleColor 
    Paint (CenterX,CenterY),NeedleColor
    Circle (CenterX,CenterY),BackWidth * 2,NeedleColor,,,,F
end sub

 

sub GraphicBar(PosX as integer,PosY as integer,BarWidth as integer,BarHeight as integer,Horizontal as integer,Frame as integer,ByRef BarColor as UInteger,MinValue as Single,MaxValue as Single,CurrentValue as Single)
    Dim NormalisedValue as Single

    if Frame<>0 then
        line (PosX,PosY) - (PosX+BarWidth,PosY+BarHeight),BarColor, b
    end if
    NormalisedValue = (CurrentValue - MinValue) / (MaxValue - MinValue)
    if NormalisedValue < 0.0 then NormalisedValue = 0.0
    if NormalisedValue > 1.0 then NormalisedValue = 1.0
    if Horizontal<>0 then
        line (PosX, PosY+BarHeight) - (PosX + (NormalisedValue * BarWidth), PosY) ,BarColor, BF
    else
        line (PosX, PosY+BarHeight) - (PosX + BarWidth, PosY + BarHeight - (NormalisedValue * BarHeight)) ,BarColor, BF
    end if
end sub

 
Sub UpdateParameters

    static as Integer i=0
    static as Integer speed_dir=2
    
    
    i=i+1
    
    if i>100 then i=0
    
    Speed = Speed+speed_dir
    if (Speed > 130) then
        speed_dir=-1
        Speed = 130
    elseif (Speed <100 ) then
        speed_dir=1
        Speed = 100

        Value_Gear=Value_Gear+1
        if Value_Gear>6 then Value_Gear=0
    end if
    
    RPM = 3000+i
    
    
    

end sub






' ********************************************************************
' * Refresh/Redraw the entire display.
' *
' ********************************************************************

sub RedrawDisplay
    
    
end sub


ScreenRes 1000,100,32, 2
ScreenSet 1,0


' font.loadfont ("ozone.xf", 1) 'Load to FontIndex 1
 'font.loadfont ("Segment.xf", 2) 'Load to FontIndex 1
'font.fontindex = 1 'Use the font

'font.backcolor = Rgba(0,255,0,255)

ReadLayouts

'Line (10,30)-(310,37), RGBA(127, 0, 0,0), BF


'font.drawstring (,"Hello World!",100,10)

'font.drawstring (,"1234567890.0",100,150,.5,.5) '2x size

'font.drawstring (,"1234567890.0",100,300,2,2) 'Vertical


'****************************************************
'Dim As Any Ptr img,img2

'img = LoadBMP( "picture.bmp" )
'img2 = LoadBMP( "dashboard.bmp" )

'If img = NULL Then
'    Print "bmp_load failed"

'Else

 '   Put (10, 10), img

'    ImageDestroy( img )

'End If

'****************************************************
'sleep 5000
Dim j as Single = 25
Dim jd as Single = 1.3
Dim NumItems as Integer
Dim CurrentLayout as Integer

CurrentLayout=1
NumItems=Layout(CurrentLayout).NumberItems

ScreenRes Layout(CurrentLayout).LayoutSizeX,Layout(CurrentLayout).LayoutSizeY,32, 2    ', 16
ScreenSet 1,0

while inkey$=""

    cls

    UpdateParameters

    for i=1 to NumItems
        
        select case Item(CurrentLayout, i).ItemType
 
            case TYPE_NEEDLE
                Needle(Item(CurrentLayout, i).ItemPosX, _
                           Item(CurrentLayout, i).ItemPosY, _
                           Item(CurrentLayout, i).ItemSize, _
                           Item(CurrentLayout, i).ItemTipWidth, _
                           Item(CurrentLayout, i).ItemBackWidth, _
                           Item(CurrentLayout, i).ItemColor, _
                           Item(CurrentLayout, i).ItemMinDeg, _
                           Item(CurrentLayout, i).ItemMaxDeg, _
                           Item(CurrentLayout, i).ItemMinValue, _
                           Item(CurrentLayout, i).ItemMaxValue, _
                           Speed)   ' Item(CurrentLayout, i).ItemParameter)
                                    

            case TYPE_STATIC_TEXT
                StaticText(Item(CurrentLayout, i).ItemPosX, _
                           Item(CurrentLayout, i).ItemPosY, _
                           Item(CurrentLayout, i).ItemSize, _
                           Item(CurrentLayout, i).ItemFName, _
                           Item(CurrentLayout, i).ItemColor, _
                           Item(CurrentLayout, i).ItemText)
                                               
            case TYPE_FORMATTED_TEXT
                FormattedValue(Item(CurrentLayout, i).ItemPosX, _
                Item(CurrentLayout, i).ItemPosY, _
                Item(CurrentLayout, i).ItemSize, _
                Item(CurrentLayout, i).ItemFName, _
                Item(CurrentLayout, i).ItemColor, _
                Item(CurrentLayout, i).ItemText, _
                Value_Gear)


 
            case TYPE_BMP_IMAGE
                Put(Item(CurrentLayout, i).ItemPosX,Item(CurrentLayout, i).ItemPosY), Item(CurrentLayout, i).ItemImagePtr, trans

 
            case TYPE_LEVEL_METER      ' Special Built in psuedo font "Shapes" that can do Circles - perhaps even have LED bitmaps?
                FontBar(Item(CurrentLayout, i).ItemPosX, _
                           Item(CurrentLayout, i).ItemPosY, _
                           Item(CurrentLayout, i).ItemSize, _
                           Item(CurrentLayout, i).ItemFName, _
                           Item(CurrentLayout, i).ItemColor1, _
                           Item(CurrentLayout, i).ItemColor2, _
                           Item(CurrentLayout, i).ItemColor3, _
                           Item(CurrentLayout, i).ItemColor4, _
                           asc(Item(CurrentLayout, i).ItemChar), _
                           Item(CurrentLayout, i).ItemText, _
                           Item(CurrentLayout, i).ItemMinValue, _
                           Item(CurrentLayout, i).ItemMaxValue, _
                           Speed)   ' Item(CurrentLayout, i).ItemParameter)
 

            case TYPE_BAR_METER

 
            case else

                                               
        end select

    next i

    flip 1,0
 
    Sleep 10

'    i=i+1
'    if i>100 then i=0
'cls
'Put (600,100), img, trans
'Put (100,450), img2
'Line (10,30)-(310,100), RGBA(127, 0, 0,255), BF

'StaticText(10,10,24,"Tahoma",Rgb(0,100,255), " Display this:")
'StaticText(EndPosX,10,24,"Tahoma",Rgb(0,100,255), "Followed by this HILL III")


'FontBar(30, 100, 64, "ozone", Rgba(0,255,255,255), asc("|"), "111112222233333444444444444444", 0.0, 100.0, i)
'    FontBar(90, 350, 64, "Tahoma", Rgba(255,100,0,255), asc("."), "1111122222333334444444444444", 0.0, 100.0, i)

'    Needle(750, 217, 90, 1,5, RGB(255,0,0), 200.0 , 190.0, 0, 100, i)

'    FormattedValue(50, 300, 24, "Consolas", RGB(0,200,0), "\T\E\S\T\ #,###.00 km/h", i*16.1334)

'    GraphicBar(20, 200, 150, 20, 1, 1, RGB(0,0,255), 20,120,i)

'    j=j+jd
'    if (j>80) or (j<20) then jd=jd*-1
    
    
'    Needle(350, 650, 100, 1,2, RGB(255,0,0), 270.0 , 90.0, -500, 600, j)
'    Needle(210, 645, 80, 1,2, RGB(255,0,0), 270.0 , 0.0, 0, 100, j)

'      FormattedValue(750, 450, 200, "Let's go Digital", RGB(0,200,0), "#", int(i/12))
'      FormattedValue(700, 450, 200, "xxWRC Clean Roman", RGB(0,200,0), "#", int(i/12))
  

'flip 1,0

'    font.drawstring (,str(i),100,250,2,2) '2x size
'   sleep 20
   
wend

'ScreenRes 500,200,32
'font.drawstring (,"Hello World!",100,10)


Sleep

' MOVE A WINDOW WITHOUT FRAMES  C#

'public const int WM_NCLBUTTONDOWN = 0xA1;
'public const int HT_CAPTION = 0x2;
' 
'[System.Runtime.InteropServices.DllImportAttribute("user32.dll")]
'public static extern int SendMessage(IntPtr hWnd,
'int Msg, int wParam, int lParam);
'[System.Runtime.InteropServices.DllImportAttribute("user32.dll")]
'public static extern bool ReleaseCapture();
' 
'private void titelLeiste_MouseDown(object sender, MouseEventArgs e)
'{
'   ReleaseCapture();
'   SendMessage(Handle, WM_NCLBUTTONDOWN, HT_CAPTION, 0);
'}








' FBGFX Font Render by Mysoft

'' DrawFont([BUFFER],[POSX],[POSY],"STRING","Font Name",FontSize,[Color],[Style],[Charset])

''uncomment the line below if you want the program to use unicode
'#define Unicode

'' this define allow you to set some effect while using anti-alias/blur (1 to 2)

' *************************************************************
' ************************ EXAMPLE ****************************
' *************************************************************

'screenres 640,480,32,5

'DrawFont(,30,20,"Testing ITALICI","Times New Roman",32,rgb(32,64,255),FS_ITALIC)
'DrawFont(,380,20,"Microsoft","Tahoma",32,rgb(32,64,255),FS_BOLD or FS_ANTIALIAS)
'DrawFont(,30,100,"Testing ITALIC heuaheuea","Script",50,rgb(32,64,255))
'DrawFont(,30,180,"Testing ITALIC heuaheuea","Script",50,rgb(32,64,255),FS_ANTIALIAS)
'DrawFont(,30,300,"","Webdings",100,rgb(32,64,255),FS_BLUR)

'sleep

' ***********************************************************************
' ***********************************************************************
' ***********************************************************************

sub DrawFont(BUFFER as any ptr=0,POSX as integer, POSY as integer, _
  FTEXT as string, FNAME as string,FSIZE as integer, _
  FCOLOR as uinteger=rgba(255,255,255,0), FSTYLE as integer=0, CHARSET as integer=DEFAULT_CHARSET )
  
  ' allocating as static for speed
  static FINIT as integer
  static as hdc THEDC
  static as hbitmap THEBMP
  static as any ptr THEPTR
  static as fb.image ptr FBBLK
  static as integer WIDCHAR(65535)
  static as integer TXTSZ,COUNT,RESU,RESUU
  static as any ptr SRCBUF,DSTBUF
  static as hfont THEFONT
  static as integer FW,FI,TXYY,FCOR
  static DSKWND as hwnd, DSKDC as hdc
  static MYBMPINFO as BITMAPINFO
  static as TEXTMETRIC MYTXINFO
  static as SIZE TXTSIZE
  static as RECT RCT
  
  #define FontSize(PointSize) -MulDiv(PointSize, GetDeviceCaps(THEDC, LOGPIXELSY), 72)  
  
  if FINIT = 0 then    
    '' allocating things and starting the "engine"
    FINIT = 1    
    with MYBMPINFO.bmiheader
      .biSize = sizeof(BITMAPINFOHEADER)
      .biWidth = 2048
      .biHeight = -513
      .biPlanes = 1
      .biBitCount = 32
      .biCompression = BI_RGB
    end with    
    ' creating a DC and a bitmap that will receive the rendered font
    DSKWND = GetDesktopWindow()
    DSKDC = GetDC(DSKWND)
    THEDC = CreateCompatibleDC(DSKDC)
    THEBMP = CreateDIBSection(THEDC,@MYBMPINFO,DIB_RGB_COLORS,@THEPTR,null,null)    
    ReleaseDC(DSKWND,DSKDC)    
  end if
  
  ' creating the font
  if (FSTYLE and FS_BOLD) then FW = FW_BOLD else FW = FW_NORMAL    
  if (FSTYLE and FS_ITALIC) then FI = True else FI = False    
  
  THEFONT = CreateFont(FontSize(FSIZE),0,0,0,FW,FI,0,0,CHARSET,0,0,0,0,cast(any ptr,strptr(FNAME)))    
  
  ' selecting it
  SelectObject(THEDC,THEBMP)
  SelectObject(THEDC,THEFONT)
  
  GetTextMetrics(THEDC,@MYTXINFO)
    
  ' get text width/height
  GetTextExtentPoint32(THEDC,strptr(FTEXT),len(FTEXT),@TXTSIZE)  
  TXTSZ = TXTSIZE.CX
  TXYY = TXTSIZE.CY
  if (FSTYLE and FS_ITALIC) then 
    if MYTXINFO.tmOverhang then
      TXTSZ += MYTXINFO.tmOverhang
    else
      TXTSZ += 1+(FSIZE/2)
    end if
    TXYY += 1+(FSIZE/8)
  end if
  if (FSTYLE and FS_ANTIALIAS) then 
    #if GAMMA>1 and GAMMA <= 2
    TXTSZ += GAMMA*2
    #endif
  end if
  with RCT
    .LEFT = 0
    .TOP = 1
    .RIGHT = TXTSZ
    .BOTTOM = TXYY+1
  end with
  TXTSZ -= 1
  TXYY -= 1
  
  ' RGB to BGR
  asm
    mov eax,[FCOLOR]
    and eax,0xFFFFFF
    mov [FCOR],eax
    bswap eax
    ror eax,8
    mov [FCOLOR],eax
  end asm
  
  ' Set Colors
  SetBkColor(THEDC,rgba(255,0,255,0))
  SetTextColor(THEDC,FCOLOR)
  
  ' disabling font smooth,rendering and enable again (if applicable)
  SystemParametersInfo(SPI_GETFONTSMOOTHING,null,@RESU,null)
 Screenlock
 if RESU then SystemParametersInfo(SPI_SETFONTSMOOTHING,False,@RESUU,null)
  ExtTextOut(THEDC,0,1,ETO_CLIPPED or ETO_OPAQUE,@RCT,strptr(FTEXT),len(FTEXT),null)
  if RESU then SystemParametersInfo(SPI_SETFONTSMOOTHING,True,@RESUU,null)
 Screenunlock  
  
  ' filling FBGFX header
  FBBLK = THEPTR+(2048*4)-sizeof(fb.image)
  FBBLK->type = 7
  FBBLK->bpp = 4
  FBBLK->width = 2048
  FBBLK->height = 512
  FBBLK->pitch = 2048*4
  
  ' blitting the rendered font to destion
  if (FSTYLE and FS_ANTIALIAS) then
    dim as any ptr MYBLK
    MYBLK = THEPTR+(2048*4)
    asm
      mov ecx,2048*511
      mov ebx,[FCOR]
      mov esi,[MYBLK]      
      HERE:
      cmp [esi], dword ptr 0xFF00FF      
      je _TRANS_
      mov [esi+3], byte ptr 0xFF      
      _TRANS_:      
      and [esi], dword ptr 0xFF000000
      or [esi], ebx
      add esi,4
      dec ecx
      jnz HERE
    end asm
    
    dim as integer TX,TY
    dim as integer ALP
    #define GetAlpha(PX,PY) peek(MYBLK+((PY)*8192)+((PX)*4)+3)
    #define SetAlpha(PX,PY,NA) poke(MYBLK+((PY)*8192)+((PX)*4)+3),NA
    
    if (FSTYLE and FS_BLUR) = FS_BLUR then
      ' blur primeira linha
      for TX = 1 to TXTSZ-1
        ALP = (GetAlpha(TX,0)+GetAlpha(TX+1,0)+GetAlpha(TX-1,0)+ _
        GetAlpha(TX,1)+GetAlpha(TX-1,1)+GetAlpha(TX+1,1)) / 6
        #if GAMMA>1 and GAMMA <= 1.6
        ALP *= (GAMMA+.5)
        if ALP > 255 then ALP = 255
        #endif
        SetAlpha(TX,TY,ALP)
      next TX
      ' blur conteudo
      for TX = 1 to TXTSZ-1
        for TY = 1 to TXYY-1          
          ALP = (GetAlpha(TX,TY)+GetAlpha(TX+1,TY)+GetAlpha(TX-1,TY)+ _
          GetAlpha(TX,TY-1)+GetAlpha(TX,TY+1) + _
          GetAlpha(TX-1,TY-1)+GetAlpha(TX-1,TY+1)+ _
          GetAlpha(TX+1,TY-1)+GetAlpha(TX+1,TY+1)) / 9
          #if GAMMA>1 and GAMMA <= 1.6
        ALP *= (GAMMA+.5)
        if ALP > 255 then ALP = 255
        #endif
          SetAlpha(TX,TY,ALP)
        next TY
      next TX
      ' blur ultima linha
      for TX = 1 to TXTSZ-1
        ALP = (GetAlpha(TX,TY)+GetAlpha(TX+1,TY)+GetAlpha(TX-1,TY)+ _
        GetAlpha(TX,TY-1)+GetAlpha(TX-1,TY-1)+GetAlpha(TX+1,TY-1)) / 6
        #if GAMMA>1 and GAMMA <= 1.6
        ALP *= (GAMMA+.5)
        if ALP > 255 then ALP = 255
        #endif
        SetAlpha(TX,TY,ALP)
      next TX
    else      
      ' antialias primeira linha
      for TX = 1 to TXTSZ-1
        ALP = (GetAlpha(TX,0)+GetAlpha(TX+1,0)+_
        GetAlpha(TX-1,0)+GetAlpha(TX,1))/4        
        #if GAMMA>1 and GAMMA <= 2
        ALP *= GAMMA
        if ALP > 255 then ALP = 255
        #endif        
        SetAlpha(TX,TY,ALP)
      next TX
      ' antialias conteudo
      for TX = 1 to TXTSZ-1
        for TY = 1 to TXYY-1
          ALP = (GetAlpha(TX,TY)+GetAlpha(TX+1,TY)+GetAlpha(TX-1,TY)+_
          GetAlpha(TX,TY-1)+GetAlpha(TX,TY+1))/5
          #if GAMMA>1 and GAMMA <= 2
          ALP *= GAMMA
          if ALP > 255 then ALP = 255
          #endif          
          SetAlpha(TX,TY,ALP)          
        next TY
      next TX
      ' antialias ultima linha
      for TX = 1 to TXTSZ-1
        ALP = (GetAlpha(TX,TY)+GetAlpha(TX+1,TY)+ _
        GetAlpha(TX-1,TY)+GetAlpha(TX,TY-1))/4        
        #if GAMMA>1 and GAMMA <= 2
        ALP *= GAMMA
        if ALP > 255 then ALP = 255
        #endif        
        SetAlpha(TX,TY,ALP)
      next TX
    end if
    
    put BUFFER,(POSX,POSY),FBBLK,(0,0)-(TXTSZ-1,TXYY),alpha
  else  
    put BUFFER,(POSX,POSY),FBBLK,(0,0)-(TXTSZ-1,TXYY),trans
  end if
  
  ' cleanning up things
  DeleteObject(THEFONT)
  EndPosX=TXTSZ+POSX
end sub



